searchNodes=[{"type":"module","title":"Kino","doc":"Client-driven interactive widgets for Livebook. Kino is the library used by Livebook to render rich and interactive outputs directly from your Elixir code. Getting started Livebook is distributed with a set of interactive tutorials and examples, including some that specifically focus on Kino. If you're just getting started, going through these is highly recommended. You can access these notebooks by starting Livebook and clicking on &quot;Learn&quot; in the sidebar. Built-in kinos Kino provides several built-in kinos. The Kino.Shorts module provides a facade to access and use most of the Kinos in this project, although you can also use their modules directly as listed in the sidebar. For user interactions, Kino.Input and Kino.Control provide a set of widgets for entering data and capturing user events. See the respective module documentation for examples. Kino also provides facilities to aid debugging, such as Kino.Process and a custom dbg() implementation that integrates with Livebook. Custom kinos Kino renders any data structure that implements the Kino.Render protocol, falling back to the Kernel.inspect/2 representation whenever an implementation is not available. You can customize how your own data structures are rendered by implementing the Kino.Render protocol. You can also implement your own kinos by writing custom JavaScript, see Kino.JS and Kino.JS.Live for more details. Packaging When publishing custom kinos and smart cells, please consider the following guidelines: prefix package name with kino_ , usually followed by the name of the integration, such as kino_vega_lite , kino_ecto namespace all modules under KinoExample , not Kino.Example . Note that official packages maintained by the Livebook team expose public APIs under Kino. , because they are essentially direct extensions of Kino and we make sure no conflicting modules exist. Unofficial packages should follow the usual Elixir conventions with respect to module names","ref":"Kino.html"},{"type":"function","title":"Kino.animate/2","doc":"Renders a kino that periodically calls the given function to render a new result. The callback receives a stream element and should return a term to be rendered. This function uses Kino.Frame as the underlying kino. It returns nothing (a non-printable result). Examples An animation is created by turning a stream of values into subsequent animation frames: Stream . interval ( 100 ) |&gt; Stream . take ( 100 ) |&gt; Kino . animate ( fn i -&gt; Kino.Markdown . new ( &quot;**Iteration: ` \#{ i } `**&quot; ) end ) Alternatively an integer may be passed as a shorthand for Stream.interval/1 : # Render new Markdown every 100ms Kino . animate ( 100 , fn i -&gt; Kino.Markdown . new ( &quot;**Iteration: ` \#{ i } `**&quot; ) end )","ref":"Kino.html#animate/2"},{"type":"function","title":"Kino.animate/3","doc":"A stateful version of animate/2 . The callback receives a stream element and the accumulated state and it should return either of: {:cont, term_to_render, state} - the continue :halt - to no longer schedule callback evaluation Examples This function is primarily useful to consume Kino.Control events: button = Kino.Control . button ( &quot;Click&quot; ) button |&gt; Kino.Control . stream ( ) |&gt; Kino . animate ( 0 , fn _event , counter -&gt; new_counter = counter + 1 md = Kino.Markdown . new ( &quot;**Clicks: ` \#{ new_counter } `**&quot; ) { :cont , md , new_counter } end )","ref":"Kino.html#animate/3"},{"type":"function","title":"Kino.async_listen/2","doc":"Same as listen/2 , except each event is processed concurrently.","ref":"Kino.html#async_listen/2"},{"type":"function","title":"Kino.configure/1","doc":"Configures Kino. The supported options are: :inspect They are discussed individually in the sections below. Inspect A keyword list containing inspect options used for printing usual evaluation results. Defaults to pretty formatting with a limit of 50 entries. To show more entries, you configure a higher limit: Kino . configure ( inspect : [ limit : 200 ] ) You can also show all entries by setting the limit to :infinity , but keep in mind that for large data structures it is memory-expensive and is not an advised configuration in this case. Instead prefer the use of IO.inspect/2 with :infinity limit when needed. See Inspect.Opts for the full list of options.","ref":"Kino.html#configure/1"},{"type":"function","title":"Kino.inspect/2","doc":"Inspects the given term as cell output. This works essentially the same as IO.inspect/2 , except it always produces colored text and respects the configuration set with configure/1 . Opposite to render/1 , it does not attempt to render the given term as a kino.","ref":"Kino.html#inspect/2"},{"type":"function","title":"Kino.interrupt!/2","doc":"Interrupts evaluation with the given message. This function raises a specific error to let Livebook known that evaluation should be stopped. The error message and a Continue button are shown to the user, who can then attempt to resolve the source of the interrupt before resuming execution. Do not use interrupt inside listeners Since interrupt!/2 aborts the execution, it cannot be used inside Kino.listen/2 or other event listeners. In such cases, you can use Kino.Frame and render any messages directly within the frame, using Kino.Text or Kino.Markdown . Examples text = Kino.Input . text ( &quot;Input&quot; ) |&gt; Kino . render ( ) |&gt; Kino.Input . read ( ) if text == &quot;&quot; do Kino . interrupt! ( :error , &quot;Input required&quot; ) end # This will not be run if the `interrupt!` is called above Kino.Markdown . new ( &quot;**You entered:** \#{ text } &quot; )","ref":"Kino.html#interrupt!/2"},{"type":"function","title":"Kino.listen/2","doc":"Starts a process that consumes a stream with fun without blocking execution. It returns the PID of the started process. The process can be terminated with Kino.terminate_child/1 . Note that events are processed by fun sequentially. If you want to process them concurrently, use async_listen/2 . Examples This function is primarily useful to consume Kino.Control events: Kino.Control . button ( &quot;Greet&quot; ) |&gt; Kino . listen ( fn event -&gt; IO . inspect ( event ) end ) You can also merge multiple controls into a single stream. For example, in order to merge them and tag each with a distinct event: button = Kino.Control . button ( &quot;Hello&quot; ) input = Kino.Input . checkbox ( &quot;Check&quot; ) stream = Kino.Control . tagged_stream ( [ hello : button , check : input ] ) Kino . listen ( stream , fn { :hello , event } -&gt; ... { :check , event } -&gt; ... end ) Any other stream works as well: Stream . interval ( 100 ) |&gt; Stream . take ( 10 ) |&gt; Kino . listen ( fn i -&gt; IO . puts ( &quot;Ping \#{ i } &quot; ) end ) Finally, an integer may be passed as a shorthand for Stream.interval/1 : Kino . listen ( 100 , fn i -&gt; IO . puts ( &quot;Ping \#{ i } &quot; ) end )","ref":"Kino.html#listen/2"},{"type":"function","title":"Kino.listen/3","doc":"A stateful version of listen/2 . The callback should return either of: {:cont, state} - the continue :halt - to stop listening Examples button = Kino.Control . button ( &quot;Click&quot; ) Kino . listen ( button , 0 , fn _event , counter -&gt; new_counter = counter + 1 IO . puts ( &quot;Clicks: \#{ new_counter } &quot; ) { :cont , new_counter } end )","ref":"Kino.html#listen/3"},{"type":"function","title":"Kino.nothing/0","doc":"Returns a special value that results in no visible output. Examples This is especially handy when you wish to suppress the default output of a cell. For instance, a cell containing this would normally result in verbose response output: resp = Req . get! ( &quot;https://example.org&quot; ) That output can be suppressed by appending a call to nothing/0 : resp = Req . get! ( &quot;https://example.org&quot; ) Kino . nothing ( )","ref":"Kino.html#nothing/0"},{"type":"function","title":"Kino.render/1","doc":"Renders the given term as cell output. This effectively allows any Livebook cell to have multiple evaluation results.","ref":"Kino.html#render/1"},{"type":"function","title":"Kino.start_child/1","doc":"Starts a process under the Kino supervisor. The process is automatically terminated when the current process terminates or the current cell reevaluates. If you want to terminate the started process, use terminate_child/1 . If you terminate the process manually, the Kino supervisor might restart it if the child's :restart strategy says so. Nested start It is not possible to use start_child/1 while initializing another process started this way. In other words, you generally cannot call start_child/1 inside callbacks such as GenServer.init/1 or Kino.JS.Live.init/2 . If you do that, starting the process will block forever. Note that creating many kinos uses start_child/1 underneath, hence the same restriction applies to starting those. See Kino.JS.Live.init/2 for more details.","ref":"Kino.html#start_child/1"},{"type":"function","title":"Kino.start_child!/1","doc":"Similar to start_child/2 but returns the new pid or raises an error.","ref":"Kino.html#start_child!/1"},{"type":"function","title":"Kino.terminate_child/1","doc":"Terminates a child started with start_child/1 . Returns :ok if the child was found and terminated, or {:error, :not_found} if the child was not found.","ref":"Kino.html#terminate_child/1"},{"type":"function","title":"Kino.tmp_dir/0","doc":"Returns a temporary directory that gets removed when the runtime terminates.","ref":"Kino.html#tmp_dir/0"},{"type":"type","title":"Kino.nothing/0","doc":"","ref":"Kino.html#t:nothing/0"},{"type":"module","title":"Kino.Control","doc":"Various widgets for user interactions. Each widget is a UI control element that the user interacts with, consequently producing an event stream. Those widgets are often useful paired with Kino.Frame for presenting content that changes upon user interactions. Examples First, create a control and make sure it is rendered, either by placing it at the end of a code cell or by explicitly rendering it with Kino.render/1 . button = Kino.Control . button ( &quot;Hello&quot; ) Next, events need to be received from the control. This can be done either by subscribing a process to the control with subscribe/2 or by creating an event stream using stream/1 or tagged_stream/1 and then registering a callback using Kino.listen/2 . Here, we'll subscribe the current process to events: Kino.Control . subscribe ( button , :hello ) As the user clicks the button, the subscribed process receives events: IEx.Helpers . flush ( ) #=&gt; {:hello, %{origin: &quot;client1&quot;}} #=&gt; {:hello, %{origin: &quot;client1&quot;}}","ref":"Kino.Control.html"},{"type":"function","title":"Kino.Control.button/1","doc":"Creates a new button. Examples Create the widget: button = Kino.Control . button ( &quot;Hello&quot; ) Listen to events: Kino . listen ( button , fn event -&gt; ... end ) Or subscribe to them in a separate process: Kino.Control . subscribe ( button , :keyboard )","ref":"Kino.Control.html#button/1"},{"type":"function","title":"Kino.Control.form/2","doc":"Creates a new form. A form is composed of regular inputs from the Kino.Input module, however in a form input values are not synchronized between users. Consequently, the form is another control for producing user-specific events. Either :submit or :report_changes must be specified. Options :submit - specifies the label to use for the submit button and enables submit events :report_changes - whether to send new form value whenever any of the input changes. Defaults to false :reset_on_submit - a list of fields to revert to their default values once the form is submitted. Use true to indicate all fields. Defaults to [] Event info In addition to standard properties, all events include additional properties. :type - either :submit or :change :data - a map with field values, matching the field list Examples Create a form out of inputs: form = Kino.Control . form ( [ name : Kino.Input . text ( &quot;Name&quot; ) , message : Kino.Input . textarea ( &quot;Message&quot; ) ] , submit : &quot;Send&quot; ) Listen to events: Kino . listen ( keyboard , fn event -&gt; ... end ) Or subscribe to them in a separate process: Kino.Control . subscribe ( form , :chat_form ) As users submit the form the payload is sent: IEx.Helpers . flush ( ) #=&gt; {:chat_form, #=&gt; %{ #=&gt; data: %{message: &quot;Hola&quot;, name: &quot;Amy&quot;}, #=&gt; origin: &quot;client1&quot;, #=&gt; type: :submit #=&gt; }} #=&gt; {:chat_form, #=&gt; %{ #=&gt; data: %{message: &quot;Hey!&quot;, name: &quot;Jake&quot;}, #=&gt; origin: &quot;client2&quot;, #=&gt; type: :submit #=&gt; }}","ref":"Kino.Control.html#form/2"},{"type":"function","title":"Kino.Control.interval/1","doc":"Returns a new interval event source. This can be used as event source for stream/1 and tagged_stream/1 . The events are emitted periodically with an increasing value, starting from 0 and have the form: %{ type : :interval , iteration : non_neg_integer ( ) }","ref":"Kino.Control.html#interval/1"},{"type":"function","title":"Kino.Control.keyboard/2","doc":"Creates a new keyboard control. This widget is represented as button that toggles interception mode, in which the given keyboard events are captured. Keyboard shortcut As of Livebook v0.11, keyboard controls can be toggled by focusing the cell and pressing ctrl + k (or ⌘ + k on MacOS). Options Note that these options require Livebook v0.11 or later. :default_handlers - controls Livebook's default keyboard shortcut handlers while the keyboard control is enabled. Must be one of: :off (default) - all Livebook keyboard shortcuts are disabled :on - all Livebook keyboard shortcuts are enabled :disable_only - Livebook keyboard shortcuts are off except for the shortcut to toggle (disable) the control Event info In addition to standard properties, all events include additional properties. Key events :type - either :keyup or :keydown :key - the value matching the browser KeyboardEvent.key Status event :type - either :status :enabled - whether the keyboard is activated Examples Create the widget: keyboard = Kino.Control . keyboard ( [ :keyup , :keydown , :status ] ) Listen to events: Kino . listen ( keyboard , fn event -&gt; ... end ) Or subscribe to them in a separate process: Kino.Control . subscribe ( keyboard , :keyboard ) As the user types events are streamed: IEx.Helpers . flush ( ) #=&gt; {:keyboard, %{enabled: true, origin: &quot;client1&quot;, type: :status} #=&gt; {:keyboard, %{key: &quot;o&quot;, origin: &quot;client1&quot;, type: :keydown}} #=&gt; {:keyboard, %{key: &quot;k&quot;, origin: &quot;client1&quot;, type: :keydown}} #=&gt; {:keyboard, %{key: &quot;o&quot;, origin: &quot;client1&quot;, type: :keyup}} #=&gt; {:keyboard, %{key: &quot;k&quot;, origin: &quot;client1&quot;, type: :keyup}}","ref":"Kino.Control.html#keyboard/2"},{"type":"function","title":"Kino.Control.stream/1","doc":"Merges several inputs and controls into a single stream of events. It accepts a single source or a list of sources, where each source is either of: %Kino.Control{} - emitting value on relevant interaction %Kino.Input{} - emitting value on value change %Kino.JS.Live{} - emitting value programmatically interval/0 - emitting value periodically, see interval/1 You can then consume the stream to access its events. The stream is typically consumed via Kino.listen/2 . Example button = Kino.Control . button ( &quot;Hello&quot; ) input = Kino.Input . checkbox ( &quot;Check&quot; ) interval = Kino.Control . interval ( 1000 ) [ button , input , interval ] |&gt; Kino.Control . stream ( ) |&gt; Kino . listen ( fn event -&gt; IO . inspect ( event ) end ) #=&gt; %{type: :interval, iteration: 0} #=&gt; %{origin: &quot;client1&quot;, type: :click} #=&gt; %{origin: &quot;client1&quot;, type: :change, value: true}","ref":"Kino.Control.html#stream/1"},{"type":"function","title":"Kino.Control.subscribe/2","doc":"Subscribes the calling process to control or input events. This is an alternative API to stream/1 , such that event messages are consumed via process messages instead of streams. The events are sent as {tag, info} , where info is a map with event details. In particular, it always includes :origin , which is an opaque identifier of the client that triggered the event.","ref":"Kino.Control.html#subscribe/2"},{"type":"function","title":"Kino.Control.tagged_stream/1","doc":"Same as stream/1 , but attaches custom tag to every stream item. Example button = Kino.Control . button ( &quot;Hello&quot; ) input = Kino.Input . checkbox ( &quot;Check&quot; ) [ hello : button , check : input ] |&gt; Kino.Control . tagged_stream ( ) |&gt; Kino . listen ( fn event -&gt; IO . inspect ( event ) end ) #=&gt; {:hello, %{origin: &quot;client1&quot;, type: :click}} #=&gt; {:check, %{origin: &quot;client1&quot;, type: :change, value: true}}","ref":"Kino.Control.html#tagged_stream/1"},{"type":"function","title":"Kino.Control.unsubscribe/1","doc":"Unsubscribes the calling process from control or input events.","ref":"Kino.Control.html#unsubscribe/1"},{"type":"type","title":"Kino.Control.event_source/0","doc":"","ref":"Kino.Control.html#t:event_source/0"},{"type":"opaque","title":"Kino.Control.interval/0","doc":"","ref":"Kino.Control.html#t:interval/0"},{"type":"opaque","title":"Kino.Control.t/0","doc":"","ref":"Kino.Control.html#t:t/0"},{"type":"module","title":"Kino.FS","doc":"Provides access to notebook files.","ref":"Kino.FS.html"},{"type":"function","title":"Kino.FS.file_path/1","doc":"Accesses notebook file with the given name and returns a local path to read its contents from. This invocation may take a while, in case the file is downloaded from a URL and is not in the cache. File operations You should treat the file as read-only. To avoid unnecessary copies the path may potentially be pointing to the original file, in which case any write operations would be persisted. This behaviour is not always the case, so you should not rely on it either.","ref":"Kino.FS.html#file_path/1"},{"type":"function","title":"Kino.FS.file_spec/1","doc":"Accesses notebook file with the given name and returns a specification of the file location. This does not copy any files and moves the responsibility of reading the file to the caller. If you need to read a file directly, use file_path/1 .","ref":"Kino.FS.html#file_spec/1"},{"type":"module","title":"Kino.Frame","doc":"A placeholder for outputs. A frame wraps outputs that can be dynamically updated at any time. Also see Kino.animate/3 which offers a convenience on top of this kino. Examples frame = Kino.Frame . new ( ) |&gt; Kino . render ( ) for i &lt;- 1 .. 100 do Kino.Frame . render ( frame , i ) Process . sleep ( 50 ) end Or with a scheduled task in the background. frame = Kino.Frame . new ( ) |&gt; Kino . render ( ) Kino . listen ( 50 , fn i -&gt; Kino.Frame . render ( frame , i ) end )","ref":"Kino.Frame.html"},{"type":"function","title":"Kino.Frame.append/3","doc":"Renders and appends the given term to the frame. Options :to - the client id to whom the update is directed. This option is useful when updating frame in response to client events, such as form submission :temporary - when true , the update is applied only to the connected clients and doesn't become a part of frame history. Defaults to false , unless :to is given. Direct updates are never a part of frame history","ref":"Kino.Frame.html#append/3"},{"type":"function","title":"Kino.Frame.clear/2","doc":"Removes all outputs within the given frame. Options :to - the client id to whom the update is directed. This option is useful when updating frame in response to client events, such as form submission :temporary - when true , the update is applied only to the connected clients and doesn't become a part of frame history. Defaults to false , unless :to is given. Direct updates are never a part of frame history","ref":"Kino.Frame.html#clear/2"},{"type":"function","title":"Kino.Frame.new/1","doc":"Creates a new frame. Options :placeholder - whether to render a placeholder when the frame is empty. Defaults to true","ref":"Kino.Frame.html#new/1"},{"type":"function","title":"Kino.Frame.render/3","doc":"Renders the given term within the frame. This works similarly to Kino.render/1 , but the rendered output replaces existing frame contents. Options :to - the client id to whom the update is directed. This option is useful when updating frame in response to client events, such as form submission :temporary - when true , the update is applied only to the connected clients and doesn't become a part of frame history. Defaults to false , unless :to is given. Direct updates are never a part of frame history","ref":"Kino.Frame.html#render/3"},{"type":"opaque","title":"Kino.Frame.t/0","doc":"","ref":"Kino.Frame.html#t:t/0"},{"type":"module","title":"Kino.Hub","doc":"Functions related to hub integrations and Livebook apps.","ref":"Kino.Hub.html"},{"type":"function","title":"Kino.Hub.app_info/0","doc":"Returns information about the running app. Note that :started_by information is only available for multi-session apps when the app uses a Livebook Teams hub. Unless called from withing an app deployment, returns %{type: :none} .","ref":"Kino.Hub.html#app_info/0"},{"type":"type","title":"Kino.Hub.app_info/0","doc":"","ref":"Kino.Hub.html#t:app_info/0"},{"type":"type","title":"Kino.Hub.user_info/0","doc":"","ref":"Kino.Hub.html#t:user_info/0"},{"type":"module","title":"Kino.Input","doc":"Various input elements for entering data. Examples First, create an input and make sure it is rendered, either by placing it at the end of a code cell or by explicitly rendering it with Kino.render/1 . input = Kino.Input . text ( &quot;Name&quot; ) Then read the value after the input has been rendered: name = Kino.Input . read ( input ) All inputs are shared by default: once you change the input, your changes will be immediately replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have their own input. Async API You can subscribe to input changes or use the Stream API for event feed. See the Kino.Control module for more details.","ref":"Kino.Input.html"},{"type":"function","title":"Kino.Input.audio/2","doc":"Creates a new audio input. The input value is a map, with an audio file and metadata: %{ file_ref : term ( ) , num_channels : pos_integer ( ) , sampling_rate : pos_integer ( ) , format : :pcm_f32 | :wav } Note that the value can also be nil , if no audio is selected. The file path can then be accessed using file_path/1 . Warning The audio input is shared by default: once you upload an audio, the audio will be replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have a distinct audio upload with an explicit submission button. Options :format - the format to read the audio as, either of: :pcm_f32 (default) - the PCM (32-bit float) format. Note that the binary uses native system endianness. Such binary can be directly converted to an Nx tensor, with no additional decoding :wav :sampling_rate - the sampling rate (samples per second) of the audio data. Defaults to 48_000","ref":"Kino.Input.html#audio/2"},{"type":"function","title":"Kino.Input.checkbox/2","doc":"Creates a new checkbox. The input value can be either true or false . Options :default - the initial input value. Defaults to false","ref":"Kino.Input.html#checkbox/2"},{"type":"function","title":"Kino.Input.color/2","doc":"Creates a new color input. The input value can be a hex color string. Options :default - the initial input value. Defaults to #6583FF :debounce - determines when input changes are emitted. When set to :blur , the change propagates when the user leaves the input. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to :blur","ref":"Kino.Input.html#color/2"},{"type":"function","title":"Kino.Input.date/2","doc":"Creates a new date input. The input is read as a %Date{} struct. Options :default - the initial input value. Defaults to nil :min - the minimum date value :max - the maximum date value","ref":"Kino.Input.html#date/2"},{"type":"function","title":"Kino.Input.file/2","doc":"Creates a new file input. The input value is a map, with a file and metadata: %{ file_ref : term ( ) , client_name : String . t ( ) } Note that the value can also be nil , if no file is selected. The file path can then be accessed using file_path/1 . Warning The file input is shared by default: once you upload a file, the file will be replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have a distinct file upload with an explicit submission button. Considerations Note that a file may be deleted in certain cases, specifically: when the file is reuploaded when used with a form and the uploading user leaves when the input is removed The deletion is not immediate and you are unlikely to run into this in practice, however theoretically file_path/1 may point to a non-existing file. Options :accept - the list of accepted file types (either extensions or MIME types) or :any . Defaults to :any Examples To read the content of currently uploaded file we would do: # [Cell 1] input = Kino.Input . file ( &quot;File&quot; ) # [Cell 2] value = Kino.Input . read ( input ) path = Kino.Input . file_path ( value . file_ref ) File . read! ( path ) And here's how we could process an asynchronous form submission: # [Cell 1] form = Kino.Control . form ( [ file : Kino.Input . file ( &quot;File&quot; ) ] , submit : &quot;Send&quot; ) # [Cell 2] form |&gt; Kino.Control . stream ( ) |&gt; Kino . listen ( fn event -&gt; path = Kino.Input . file_path ( event . data . file . file_ref ) content = File . read! ( path ) IO . inspect ( content ) end )","ref":"Kino.Input.html#file/2"},{"type":"function","title":"Kino.Input.file_path/1","doc":"Returns file path for the given file identifier.","ref":"Kino.Input.html#file_path/1"},{"type":"function","title":"Kino.Input.image/2","doc":"Creates a new image input. The input value is a map, with an image file and metadata: %{ file_ref : term ( ) , height : pos_integer ( ) , width : pos_integer ( ) , format : :rgb | :png | :jpeg } Note that the value can also be nil , if no image is selected. The file path can then be accessed using file_path/1 . Warning The image input is shared by default: once you upload a image, the image will be replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have a distinct image upload with an explicit submission button. Options :format - the format to read the image as, either of: :rgb (default) - the binary includes raw pixel values, each encoded as a single byte in the HWC order. Such binary can be directly converted to an Nx tensor, with no additional decoding :png :jpeg (or :jpg ) :size - the size to fit the image into, given as {height, width} :fit - the strategy of fitting the image into :size , either of: :contain (default) - resizes the image, such that it fits in a box of :size , but preserving the aspect ratio. The resulting image can be smaller or equal to :size :match - resizes the image to :size , with no respect for aspect ratio :pad - same as :contain , but pads the image to match :size exactly :crop - resizes the image, such that one edge fits in :size and the other overflows, then center-crops the image to match :size exactly","ref":"Kino.Input.html#image/2"},{"type":"function","title":"Kino.Input.number/2","doc":"Creates a new number input. The input value is can be either a number or nil . Options :default - the initial input value. Defaults to nil :debounce - determines when input changes are emitted. When set to :blur , the change propagates when the user leaves the input. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to :blur","ref":"Kino.Input.html#number/2"},{"type":"function","title":"Kino.Input.password/2","doc":"Creates a new password input. This is similar to text input, except the content is not visible by default. Options :default - the initial input value. Defaults to &quot;&quot; :debounce - determines when input changes are emitted. When set to :blur , the change propagates when the user leaves the input. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to :blur","ref":"Kino.Input.html#password/2"},{"type":"function","title":"Kino.Input.range/2","doc":"Creates a new slider input. The input value can be either float in the configured range. Options :default - the initial input value. Defaults to the minimum value :min - the minimum value :max - the maximum value :step - the slider increment :debounce - determines when input changes are emitted. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to 250","ref":"Kino.Input.html#range/2"},{"type":"function","title":"Kino.Input.read/1","doc":"Synchronously reads the current input value. Examples input = Kino.Input . text ( &quot;Name&quot; ) |&gt; Kino . render ( ) Kino.Input . read ( input )","ref":"Kino.Input.html#read/1"},{"type":"function","title":"Kino.Input.select/3","doc":"Creates a new select input. The input expects a list of options in the form [{value, label}] , where value is an arbitrary term and label is a descriptive string. Options :default - the initial input value. Defaults to the first value from the given list of options Examples Kino.Input . select ( &quot;Language&quot; , [ en : &quot;English&quot; , fr : &quot;Français&quot; ] ) Kino.Input . select ( &quot;Language&quot; , [ { 1 , &quot;One&quot; } , { 2 , &quot;Two&quot; } , { 3 , &quot;Three&quot; } ] )","ref":"Kino.Input.html#select/3"},{"type":"function","title":"Kino.Input.text/2","doc":"Creates a new text input. Options :default - the initial input value. Defaults to &quot;&quot; :debounce - determines when input changes are emitted. When set to :blur , the change propagates when the user leaves the input. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to :blur","ref":"Kino.Input.html#text/2"},{"type":"function","title":"Kino.Input.textarea/2","doc":"Creates a new multiline text input. Options :default - the initial input value. Defaults to &quot;&quot; :monospace - whether to use a monospace font inside the textarea. Defaults to false :debounce - determines when input changes are emitted. When set to :blur , the change propagates when the user leaves the input. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to :blur","ref":"Kino.Input.html#textarea/2"},{"type":"function","title":"Kino.Input.url/2","doc":"Creates a new URL input. The input value can be either a valid URL string or nil . Options :default - the initial input value. Defaults to nil :debounce - determines when input changes are emitted. When set to :blur , the change propagates when the user leaves the input. When set to a non-negative number of milliseconds, the change propagates after the specified delay. Defaults to :blur","ref":"Kino.Input.html#url/2"},{"type":"function","title":"Kino.Input.utc_datetime/2","doc":"Creates a new datetime input. The input is editable in user-local time zone, however the value is always read in UTC as a %NaiveDateTime{} struct. Options :default - the initial input value. Defaults to nil :min - the minimum datetime value (in UTC) :max - the maximum datetime value (in UTC)","ref":"Kino.Input.html#utc_datetime/2"},{"type":"function","title":"Kino.Input.utc_time/2","doc":"Creates a new time input. The input is editable in user-local time zone, however the value is always read in UTC as a %Time{} struct. Options :default - the initial input value. Defaults to nil :min - the minimum time value (in UTC) :max - the maximum time value (in UTC)","ref":"Kino.Input.html#utc_time/2"},{"type":"opaque","title":"Kino.Input.t/0","doc":"","ref":"Kino.Input.html#t:t/0"},{"type":"module","title":"Kino.Process","doc":"This module contains kinos for generating visualizations to help introspect your running processes.","ref":"Kino.Process.html"},{"type":"function","title":"Kino.Process.app_tree/2","doc":"Generates a visualization of an application tree. Given the name of an application as an atom, this function will render the application tree. It is displayed with solid lines denoting supervisor-worker relationships and dotted lines denoting links between processes. The graph rendering supports the following options: Options :direction - defines the direction of the graph visual. The value can either be :top_down or :left_right . Defaults to :top_down . Examples To view the applications running in your instance run: :application_controller . which_applications ( ) You can then call Kino.Process.app_tree/1 to render the application tree using using the atom of the application. Kino.Process . app_tree ( :logger ) You can also change the direction of the rendering by calling Kino.Process.app_tree/2 with the :direction option. Kino.Process . app_tree ( :logger , direction : :left_right )","ref":"Kino.Process.html#app_tree/2"},{"type":"function","title":"Kino.Process.render_app_tree/2","doc":"Renders a visual of the provided application tree. This function renders an application tree much like app_tree/2 with the difference being that this function can be called anywhere within the Livebook code block whereas app_tree/2 must have its result be the last thing returned from the code block in order to render the visual. It supports the same options as app_tree/2 as it delegates to that function to generate the visual.","ref":"Kino.Process.html#render_app_tree/2"},{"type":"function","title":"Kino.Process.render_seq_trace/2","doc":"Renders a sequence diagram of process messages and returns the function result. This function renders a sequence diagram much like seq_trace/2 with the difference being that this function can be called anywhere within the Livebook code block whereas seq_trace/2 must have its result be the last thing returned from the code block in order to render the visual. In addition, this function returns the result from the provided trace function.","ref":"Kino.Process.html#render_seq_trace/2"},{"type":"function","title":"Kino.Process.render_sup_tree/2","doc":"Renders a visual of the provided supervision tree. This function renders a supervision tree much like sup_tree/2 with the difference being that this function can be called anywhere within the Livebook code block whereas sup_tree/2 must have its result be the last thing returned from the code block in order to render the visual. It supports the same options as sup_tree/2 as it delegates to that function to generate the visual.","ref":"Kino.Process.html#render_sup_tree/2"},{"type":"function","title":"Kino.Process.seq_trace/2","doc":"Generate a sequence diagram of process messages starting from self() . The provided function is executed and traced, with all the events sent to and received by the trace target processes rendered in a sequence diagram. The trace target argument can either be a single PID, a list of PIDs, or the atom :all depending on what messages you would like to retain in your trace. Examples To generate a trace of all the messages occurring during the execution of the provided function, you can do the following: Kino.Process . seq_trace ( fn -&gt; { :ok , agent_pid } = Agent . start_link ( fn -&gt; [ ] end ) Process . monitor ( agent_pid ) 1 .. 2 |&gt; Task . async_stream ( fn value -&gt; Agent . get ( agent_pid , fn value -&gt; value end ) 100 * value end , max_concurrency : 3 ) |&gt; Stream . run ( ) Agent . stop ( agent_pid ) end ) If you are only interested in messages being sent to or received by certain PIDs, you can filter the sequence diagram by specifying the PIDs that you are interested in: { :ok , agent_pid } = Agent . start_link ( fn -&gt; [ ] end ) Process . monitor ( agent_pid ) Kino.Process . seq_trace ( agent_pid , fn -&gt; 1 .. 2 |&gt; Task . async_stream ( fn value -&gt; Agent . get ( agent_pid , fn value -&gt; value end ) 100 * value end , max_concurrency : 3 ) |&gt; Stream . run ( ) Agent . stop ( agent_pid ) end )","ref":"Kino.Process.html#seq_trace/2"},{"type":"function","title":"Kino.Process.sup_tree/2","doc":"Generates a visualization of a supervision tree. The provided supervisor can be either a named process or a PID. The supervision tree is displayed with solid lines denoting supervisor-worker relationships and dotted lines denoting links between processes. The graph rendering supports the following options: Options :direction - defines the direction of the graph visual. The value can either be :top_down or :left_right . Defaults to :top_down . Examples With a supervisor definition like so: { :ok , supervisor_pid } = Supervisor . start_link ( [ { DynamicSupervisor , strategy : :one_for_one , name : MyApp.DynamicSupervisor } , { Agent , fn -&gt; [ ] end } ] , strategy : :one_for_one , name : MyApp.Supervisor ) Enum . each ( 1 .. 3 , fn _ -&gt; DynamicSupervisor . start_child ( MyApp.DynamicSupervisor , { Agent , fn -&gt; %{ } end } ) end ) You can then call Kino.Process.sup_tree/1 to render the supervision tree using using the PID of the supervisor. Kino.Process . sup_tree ( supervisor_pid ) You can also render the supervisor by passing the name of the supervisor if the supervisor was started with a name. Kino.Process . sup_tree ( MyApp.Supervisor ) You can also change the direction of the rendering by calling Kino.Process.sup_tree/2 with the :direction option. Kino.Process . sup_tree ( MyApp.Supervisor , direction : :left_right )","ref":"Kino.Process.html#sup_tree/2"},{"type":"type","title":"Kino.Process.supervisor/0","doc":"","ref":"Kino.Process.html#t:supervisor/0"},{"type":"type","title":"Kino.Process.trace_target/0","doc":"","ref":"Kino.Process.html#t:trace_target/0"},{"type":"module","title":"Kino.Shorts","doc":"Shortcuts for building Kinos. This module provide an easy to use Kino API and is meant to be imported into your notebooks: import Kino.Shorts","ref":"Kino.Shorts.html"},{"type":"function","title":"Kino.Shorts.audio/2","doc":"Renders an audio of any given format. It is a wrapper around Kino.Audio.new/2 . Examples import Kino.Shorts content = File . read! ( &quot;/path/to/audio.wav&quot; ) audio ( content , :wav )","ref":"Kino.Shorts.html#audio/2"},{"type":"function","title":"Kino.Shorts.data_table/2","doc":"Renders a data table output for user-provided tabular data. The data must implement the Table.Reader protocol. This function is a wrapper around Kino.DataTable.new/1 . Examples import Kino.Shorts data = [ %{ id : 1 , name : &quot;Elixir&quot; , website : &quot;https://elixir-lang.org&quot; } , %{ id : 2 , name : &quot;Erlang&quot; , website : &quot;https://www.erlang.org&quot; } ] data_table ( data )","ref":"Kino.Shorts.html#data_table/2"},{"type":"function","title":"Kino.Shorts.download/2","doc":"Renders a file download button. The given function is invoked to generate the file content whenever a download is requested. It is a wrapper around Kino.Download.new/2 . Options :filename - the default filename suggested for download. Defaults to &quot;download&quot; :label - the button text. Defaults to the value of :filename if present and &quot;Download&quot; otherwise Examples download ( fn -&gt; &quot;Example text&quot; end ) download ( fn -&gt; Jason . encode! ( %{ &quot;foo&quot; =&gt; &quot;bar&quot; } ) end , filename : &quot;data.json&quot; ) download ( fn -&gt; &lt;&lt; 0 , 1 &gt;&gt; end , filename : &quot;data.bin&quot; , label : &quot;Binary data&quot; )","ref":"Kino.Shorts.html#download/2"},{"type":"function","title":"Kino.Shorts.frame/1","doc":"A placeholder for static outputs that can be dynamically updated. The frame can be updated with the Kino.Frame module API. Also see Kino.animate/3 . Examples import Kino.Shorts frame = frame ( ) |&gt; Kino . render ( ) for i &lt;- 1 .. 100 do Kino.Frame . render ( frame , i ) Process . sleep ( 50 ) end","ref":"Kino.Shorts.html#frame/1"},{"type":"function","title":"Kino.Shorts.grid/2","doc":"Arranges outputs into a grid. Note that the grid does not support scrolling, it always squeezes the content, such that it does not exceed the page width. It is a wrapper around Kino.Layout.grid/2 . See the linked function for all supported options. Options :columns - the number of columns in the grid. Defaults to 1 :boxed - whether the grid should be wrapped in a bordered box. Defaults to false :gap - the amount of spacing between grid items in pixels. Defaults to 8 Examples import Kino.Shorts images = for path &lt;- paths do path |&gt; File . read! ( ) |&gt; image ( :jpeg ) end grid ( images , columns : 3 )","ref":"Kino.Shorts.html#grid/2"},{"type":"function","title":"Kino.Shorts.html/1","doc":"Displays arbitrary static HTML. It is a wrapper around Kino.HTML.new/1 . Examples import Kino.Shorts html ( &quot;&quot;&quot; &lt;h3&gt;Look!&lt;/h3&gt; &lt;p&gt;I wrote this HTML from &lt;strong&gt;Kino&lt;/strong&gt;!&lt;/p&gt; &quot;&quot;&quot; )","ref":"Kino.Shorts.html#html/1"},{"type":"function","title":"Kino.Shorts.image/2","doc":"Renders an image of any given format. It is a wrapper around Kino.Image.new/2 . Examples import Kino.Shorts content = File . read! ( &quot;/path/to/image.jpeg&quot; ) image ( content , &quot;image/jpeg&quot; )","ref":"Kino.Shorts.html#image/2"},{"type":"function","title":"Kino.Shorts.markdown/1","doc":"Renders Markdown content, in case you need richer text. It is a wrapper around Kino.Markdown.new/1 . Examples import Kino.Shorts markdown ( &quot;&quot;&quot; # Example A regular Markdown file. ## Code ```elixir &quot;Elixir&quot; |&gt; String.graphemes() |&gt; Enum.frequencies() ``` ## Table | ID | Name | Website | | -- | ------ | ----------------------- | | 1 | Elixir | https://elixir-lang.org | | 2 | Erlang | https://www.erlang.org | &quot;&quot;&quot; )","ref":"Kino.Shorts.html#markdown/1"},{"type":"function","title":"Kino.Shorts.mermaid/1","doc":"Renders Mermaid graphs. It is a wrapper around Kino.Mermaid.new/1 . Examples import Kino.Shorts mermaid ( &quot;&quot;&quot; graph TD; A--&gt;B; A--&gt;C; B--&gt;D; C--&gt;D; &quot;&quot;&quot; )","ref":"Kino.Shorts.html#mermaid/1"},{"type":"function","title":"Kino.Shorts.read_audio/2","doc":"Renders and reads a new audio input. See Kino.Input.audio/2 for all supported formats and options. Warning The audio input is shared by default: once you upload an audio, the audio will be replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have a distinct audio upload with an explicit submission button.","ref":"Kino.Shorts.html#read_audio/2"},{"type":"function","title":"Kino.Shorts.read_checkbox/2","doc":"Renders and reads a new checkbox. Options :default - the initial input value. Defaults to false","ref":"Kino.Shorts.html#read_checkbox/2"},{"type":"function","title":"Kino.Shorts.read_color/2","doc":"Renders and reads a new color input. Options :default - the initial input value. Defaults to #6583FF","ref":"Kino.Shorts.html#read_color/2"},{"type":"function","title":"Kino.Shorts.read_date/2","doc":"Renders and reads a new date input. Options :default - the initial input value. Defaults to nil :min - the minimum date value :max - the maximum date value","ref":"Kino.Shorts.html#read_date/2"},{"type":"function","title":"Kino.Shorts.read_file/2","doc":"Renders and reads a new file input. The file path can then be accessed using Kino.Input.file_path/1 . See Kino.Input.file/2 for additional considerations. Warning The file input is shared by default: once you upload a file, the file will be replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have a distinct file upload with an explicit submission button. Options :accept - the list of accepted file types (either extensions or MIME types) or :any . Defaults to :any","ref":"Kino.Shorts.html#read_file/2"},{"type":"function","title":"Kino.Shorts.read_image/2","doc":"Renders and reads a new image input. See Kino.Input.image/2 for all supported formats and options. Warning The image input is shared by default: once you upload a image, the image will be replicated to all users reading the notebook. Use Kino.Control.form/2 if you want each user to have a distinct image upload with an explicit submission button.","ref":"Kino.Shorts.html#read_image/2"},{"type":"function","title":"Kino.Shorts.read_number/2","doc":"Renders and reads a new number input. Options :default - the initial input value. Defaults to nil","ref":"Kino.Shorts.html#read_number/2"},{"type":"function","title":"Kino.Shorts.read_password/2","doc":"Renders and reads a new password input. Options :default - the initial input value. Defaults to &quot;&quot;","ref":"Kino.Shorts.html#read_password/2"},{"type":"function","title":"Kino.Shorts.read_range/2","doc":"Renders and reads a new slider input. Options :default - the initial input value. Defaults to the minimum value :min - the minimum value :max - the maximum value :step - the slider increment","ref":"Kino.Shorts.html#read_range/2"},{"type":"function","title":"Kino.Shorts.read_select/3","doc":"Renders and reads a new select input. The input expects a list of options in the form [{value, label}] , where value is an arbitrary term and label is a descriptive string. Options :default - the initial input value. Defaults to the first value from the given list of options Examples read_select ( &quot;Language&quot; , [ en : &quot;English&quot; , fr : &quot;Français&quot; ] ) read_select ( &quot;Language&quot; , [ { 1 , &quot;One&quot; } , { 2 , &quot;Two&quot; } , { 3 , &quot;Three&quot; } ] )","ref":"Kino.Shorts.html#read_select/3"},{"type":"function","title":"Kino.Shorts.read_text/2","doc":"Renders and reads a new text input. Options :default - the initial input value. Defaults to &quot;&quot;","ref":"Kino.Shorts.html#read_text/2"},{"type":"function","title":"Kino.Shorts.read_textarea/2","doc":"Renders and reads a new multiline text input. Options :default - the initial input value. Defaults to &quot;&quot; :monospace - whether to use a monospace font inside the textarea. Defaults to false","ref":"Kino.Shorts.html#read_textarea/2"},{"type":"function","title":"Kino.Shorts.read_url/2","doc":"Renders and reads a new URL input. Options :default - the initial input value. Defaults to nil","ref":"Kino.Shorts.html#read_url/2"},{"type":"function","title":"Kino.Shorts.read_utc_datetime/2","doc":"Renders and reads a new datetime input. Options :default - the initial input value. Defaults to nil :min - the minimum datetime value (in UTC) :max - the maximum datetime value (in UTC)","ref":"Kino.Shorts.html#read_utc_datetime/2"},{"type":"function","title":"Kino.Shorts.read_utc_time/2","doc":"Renders and reads a new time input. Options :default - the initial input value. Defaults to nil :min - the minimum time value (in UTC) :max - the maximum time value (in UTC)","ref":"Kino.Shorts.html#read_utc_time/2"},{"type":"function","title":"Kino.Shorts.tabs/1","doc":"Arranges outputs into separate tabs. It is a wrapper around Kino.Layout.tabs/1 . Examples import Kino.Shorts data = [ %{ id : 1 , name : &quot;Elixir&quot; , website : &quot;https://elixir-lang.org&quot; } , %{ id : 2 , name : &quot;Erlang&quot; , website : &quot;https://www.erlang.org&quot; } ] tabs ( [ Table : data_table ( data ) , Raw : data ] )","ref":"Kino.Shorts.html#tabs/1"},{"type":"function","title":"Kino.Shorts.text/1","doc":"Renders plain text content. It is similar to markdown/1 , however doesn't interpret any markup. It is a wrapper around Kino.Text.new/1 . Examples import Kino.Shorts text ( &quot;Hello!&quot; )","ref":"Kino.Shorts.html#text/1"},{"type":"function","title":"Kino.Shorts.tree/1","doc":"Displays arbitrarily nested data structure as a tree view. It is a wrapper around Kino.Tree.new/1 . Examples import Kino.Shorts tree ( Process . info ( self ( ) ) )","ref":"Kino.Shorts.html#tree/1"},{"type":"function","title":"Kino.Shorts.video/2","doc":"Renders a video of any given format. It is a wrapper around Kino.Audio.new/2 . Examples import Kino.Shorts content = File . read! ( &quot;/path/to/video.mp4&quot; ) video ( content , :mp4 )","ref":"Kino.Shorts.html#video/2"},{"type":"module","title":"Kino.JS","doc":"Allows for defining custom JavaScript powered kinos. Example Here's how we could define a minimal kino that embeds the given HTML directly into the page. defmodule KinoDocs.HTML do use Kino.JS def new ( html ) do Kino.JS . new ( __MODULE__ , html ) end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, html) { ctx.root.innerHTML = html; } &quot;&quot;&quot; end end Let's break down the API. To define a custom kino we need to create a new module. In this case we go with KinoDocs.HTML . We start by adding use Kino.JS , which makes our module asset-aware. In particular, it allows us to use the asset/2 macro to define arbitrary files directly in the module source. All kinos require a main.js file that defines a JavaScript module and becomes the entrypoint on the client side. The JavaScript module is expected to export the init(ctx, data) function, where ctx is a special object (discussed in detail later) and data is the kino data passed from the Elixir side. In our example the init function accesses the root element with ctx.root and overrides its content with the given HTML string. Finally, we define the new(html) function that creates kinos with the given HTML. Underneath we call Kino.JS.new/2 specifying our module as the kino type and passing the data (available in the JavaScript init function later). Again, it's a convention for each kino module to define a new function to provide uniform experience for the end user. Assets We already saw how to define a JavaScript (or any other) file using the asset/2 macro, however in most cases it's preferable to put assets in a dedicated directory to benefit from syntax highlighting and other editor features. To do that, we just need to specify where the corresponding directory is located: use Kino.JS , assets_path : &quot;lib/assets/html&quot; Stylesheets The ctx.importCSS(url) function allows us to load CSS from the given URL into the page. The stylesheet can be an external resource, such as a font from Google Fonts or a custom asset (as outlined above). Here's an example of both: defmodule KinoDocs.HTML do use Kino.JS def new ( html ) do Kino.JS . new ( __MODULE__ , html ) end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, html) { ctx.importCSS(&quot;https://fonts.googleapis.com/css?family=Sofia&quot;) ctx.importCSS(&quot;main.css&quot;) ctx.root.innerHTML = html; } &quot;&quot;&quot; end asset &quot;main.css&quot; do &quot;&quot;&quot; body { font-family: &quot;Sofia&quot;, sans-serif; } &quot;&quot;&quot; end end URLs When using multiple asset files, make sure to use relative URLs. For example, when adding an image to the page, instead of: &lt; img src = &quot;/images/cat.jpeg&quot; / &gt; Do: &lt; img src = &quot;./images/cat.jpeg&quot; / &gt; This will correctly point to the images/cat.jpeg file in your assets. Security Note that all assets are assumed public and Livebook doesn't enforce authentication when loading them. Therefore, never include any sensitive credentials in the assets source, instead pass them as arguments from your Elixir code. JavaScript API In the example we briefly introduced the ctx (context) object that is made available in the init(ctx, data) function. This object encapsulates all of the Livebook-specific API that we can call on the JavaScript side. Properties ctx.root - the root element controlled by the kino Functions ctx.importCSS(url) - loads CSS from the given URL into the page. Returns a Promise that resolves once the CSS is loaded ctx.importJS(url) - loads JS from the given URL into the page using a regular &lt;script&gt; tag. Returns a Promise that resolves once the JS is loaded ctx.handleEvent(event, callback) - registers an event handler. Once event is broadcasted, callback is executed with the event payload. This applies to Kino.JS.Live kinos ctx.pushEvent(event, payload) - sends an event to the kino server, where it is handled with Kino.JS.Live.handle_event/3 . This applies to Kino.JS.Live kinos ctx.handleSync(callback) - registers a synchronization handler, it should flush any deferred UI changes to the server. This applies to Kino.SmartCell cells ctx.selectSecret(callback, preselectName) - asks the user to select a Livebook secret. Suggests preselectName as the default choice. When the user selects a secret, callback is called with the secret name CDN It is possible to use a regular JavaScript bundler for generating the assets, however in many cases a simpler and preferred approach is to import the necessary dependencies directly from a CDN. To give a concrete example, here's how we could use the mermaid JavaScript package for rendering diagrams: defmodule KinoDocs.Mermaid do use Kino.JS def new ( graph ) do Kino.JS . new ( __MODULE__ , graph ) end asset &quot;main.js&quot; do &quot;&quot;&quot; import &quot;https://cdn.jsdelivr.net/npm/mermaid@9.1.3/dist/mermaid.min.js&quot;; mermaid.initialize({ startOnLoad: false }); export function init(ctx, graph) { mermaid.render(&quot;graph1&quot;, graph, (svgSource, bindListeners) =&gt; { ctx.root.innerHTML = svgSource; bindListeners &amp;&amp; bindListeners(ctx.root); }); } &quot;&quot;&quot; end end And we would use it like so: KinoDocs.Mermaid . new ( &quot;&quot;&quot; graph TD; A--&gt;B; A--&gt;C; B--&gt;D; C--&gt;D; &quot;&quot;&quot; ) Live kinos So far we covered the API for defining static kinos, where the JavaScript side only receives the initial data and there is no further interaction with the Elixir side. To introduce such interaction, see Kino.JS.Live as a next step in our discussion.","ref":"Kino.JS.html"},{"type":"macro","title":"Kino.JS.asset/2","doc":"Defines an asset file. This serves as a convenience when prototyping or building simple kinos, otherwise you most likely want to put assets in separate files. See the Assets for more details. Examples asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, data) { ... } &quot;&quot;&quot; end asset &quot;main.css&quot; do &quot;&quot;&quot; .box { ... } &quot;&quot;&quot; end","ref":"Kino.JS.html#asset/2"},{"type":"function","title":"Kino.JS.new/3","doc":"Instantiates a static JavaScript kino defined by module . The given data is passed directly to the JavaScript side during initialization. Options :export - a function called to export the given kino to Markdown. See the &quot;Export&quot; section below Export The output can optionally be exported in notebook source by specifying an :export function. The function receives data as an argument and should return a tuple {info_string, payload} . info_string is used to annotate the Markdown code block where the output is persisted. payload is the value persisted in the code block. The value is automatically serialized to JSON, unless it is already a string. For example: data = &quot;graph TD;A--&gt;B;&quot; Kino.JS . new ( __MODULE__ , data , export : fn data -&gt; { &quot;mermaid&quot; , data } end ) Would be rendered as the following Live Markdown: ```mermaid graph TD;A--&gt;B; ``` Export function You should prefer to use the data argument for computing the export payload. However, if it cannot be inferred from data , you should just reference the original value. Do not put additional fields in data , just to use it for export.","ref":"Kino.JS.html#new/3"},{"type":"opaque","title":"Kino.JS.t/0","doc":"","ref":"Kino.JS.html#t:t/0"},{"type":"behaviour","title":"Kino.JS.Live","doc":"Introduces state and event-driven capabilities to JavaScript powered kinos. Make sure to read the introduction to JavaScript kinos in Kino.JS for more context. Similarly to static kinos, live kinos involve a custom JavaScript code running in the browser. In fact, this part of the API is the same. In addition, each live kino has a server process running on the Elixir side, responsible for maintaining state and able to communicate with the JavaScript side at any time. Again, to illustrate the ideas we start with a minimal example. Example We will follow up on our KinoDocs.HTML example by adding support for replacing the content on demand. defmodule KinoDocs.LiveHTML do use Kino.JS use Kino.JS.Live def new ( html ) do Kino.JS.Live . new ( __MODULE__ , html ) end def replace ( kino , html ) do Kino.JS.Live . cast ( kino , { :replace , html } ) end @impl true def init ( html , ctx ) do { :ok , assign ( ctx , html : html ) } end @impl true def handle_connect ( ctx ) do { :ok , ctx . assigns . html , ctx } end @impl true def handle_cast ( { :replace , html } , ctx ) do broadcast_event ( ctx , &quot;replace&quot; , html ) { :noreply , assign ( ctx , html : html ) } end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, html) { ctx.root.innerHTML = html; ctx.handleEvent(&quot;replace&quot;, (html) =&gt; { ctx.root.innerHTML = html; }); } &quot;&quot;&quot; end end Just as before we define a module, this time calling it KinoDocs.LiveHTML for clarity. Note many similarities to the previous version, we still call use Kino.JS , define the main.js file and define the new(html) function for creating a kino instance. As a matter of fact, the initial result of KinoDocs.LiveHTML.new(html) will render exactly the same as our previous KinoDocs.HTML.new(html) . As for the new bits, we added use Kino.JS.Live to define a live kino server. We use Kino.JS.Live.new/2 for creating the kino instance and we implement a few GenServer -like callbacks. Once the kino server is started with Kino.JS.Live.new/2 , the init/2 callback is called with the initial argument. In this case we store the given html in server state. Whenever the kino is rendered on a new client, the handle_connect/1 callback is called and it builds the initial data for the client. In this case, we always return the stored html . This initial data is then passed to the JavaScript init function. Keep in mind that while the server is initialized once, connect may happen at any point, as the users join/refresh the page. Finally, the whole point of our example is the ability to replace the HTML content directly from the Elixir side and for this purpose we added the public replace(kino, html) function. Underneath the function uses cast/2 to message our server and the message is handled with handle_cast/2 . In this case we store the new html in the server state and broadcast an event with the new value. On the client side, we subscribe to those events with ctx.handleEvent(event, callback) to update the page accordingly. Event handlers You must eventually register JavaScript handlers for all events that the client may receive. However, the registration can be deferred, if the initialization is asynchronous. For example, the following is perfectly fine: export function init(ctx, data) { fetch(data.someUrl).then((resp) =&gt; { ctx.handleEvent(&quot;update&quot;, (payload) =&gt; { // ... }); }); } Or alternatively: export async function init(ctx, data) { const response = await fetch(data.someUrl); ctx.handleEvent(&quot;update&quot;, (payload) =&gt; { // ... }); } In such case all incoming events are buffered and dispatched once the handler is registered. Binary payloads The client-server communication supports binary data, both on initialization and on custom events. On the server side, a binary payload has the form of {:binary, info, binary} , where info is regular JSON-serializable data that can be sent alongside the plain binary. On the client side, a binary payload is represented as [info, buffer] , where info is the additional data and buffer is the binary as ArrayBuffer . The following example showcases how to send and receive events with binary payloads. defmodule KinoDocs.Binary do use Kino.JS use Kino.JS.Live def new ( ) do Kino.JS.Live . new ( __MODULE__ , nil ) end @impl true def handle_connect ( ctx ) do payload = { :binary , %{ message : &quot;hello&quot; } , &lt;&lt; 1 , 2 &gt;&gt; } { :ok , payload , ctx } end @impl true def handle_event ( &quot;ping&quot; , { :binary , _info , binary } , ctx ) do reply_payload = { :binary , %{ message : &quot;pong&quot; } , &lt;&lt; 1 , 2 , binary :: binary &gt;&gt; } broadcast_event ( ctx , &quot;pong&quot; , reply_payload ) { :noreply , ctx } end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, payload) { console.log(&quot;initial data&quot;, payload); ctx.handleEvent(&quot;pong&quot;, ([info, buffer]) =&gt; { console.log(&quot;event data&quot;, [info, buffer]) }); const buffer = new ArrayBuffer(2); const bytes = new Uint8Array(buffer); bytes[0] = 4; bytes[1] = 250; ctx.pushEvent(&quot;ping&quot;, [{ message: &quot;ping&quot; }, buffer]); } &quot;&quot;&quot; end end","ref":"Kino.JS.Live.html"},{"type":"function","title":"Kino.JS.Live.call/3","doc":"Makes a synchronous call to the kino server and waits for its reply. See GenServer.call/3 for more details.","ref":"Kino.JS.Live.html#call/3"},{"type":"function","title":"Kino.JS.Live.cast/2","doc":"Sends an asynchronous request to the kino server. See GenServer.cast/2 for more details.","ref":"Kino.JS.Live.html#cast/2"},{"type":"callback","title":"Kino.JS.Live.handle_call/3","doc":"Invoked to handle synchronous call/3 messages. See GenServer.handle_call/3 for more details.","ref":"Kino.JS.Live.html#c:handle_call/3"},{"type":"callback","title":"Kino.JS.Live.handle_cast/2","doc":"Invoked to handle asynchronous cast/2 messages. See GenServer.handle_cast/2 for more details.","ref":"Kino.JS.Live.html#c:handle_cast/2"},{"type":"callback","title":"Kino.JS.Live.handle_connect/1","doc":"Invoked whenever a new client connects to the server. The returned data is passed to the JavaScript init function of the connecting client.","ref":"Kino.JS.Live.html#c:handle_connect/1"},{"type":"callback","title":"Kino.JS.Live.handle_event/3","doc":"Invoked to handle client events.","ref":"Kino.JS.Live.html#c:handle_event/3"},{"type":"callback","title":"Kino.JS.Live.handle_info/2","doc":"Invoked to handle all other messages. See GenServer.handle_info/2 for more details.","ref":"Kino.JS.Live.html#c:handle_info/2"},{"type":"callback","title":"Kino.JS.Live.init/2","doc":"Invoked when the server is started. See GenServer.init/1 for more details. Starting other kinos It is generally not possible to start kinos inside the init/2 callback, as such operation would block forever. In case you need to start other kinos during initialization, you must start them beforehand and pass as an argument to init/2 . So instead of defmodule KinoDocs.Custom do def new ( ) do Kino.JS.Live . new ( __MODULE__ , { } ) end @impl true def init ( { } , ctx ) do frame = Kino.Frame . new ( ) { :ok , assign ( ctx , frame : frame ) } end ... end do the following defmodule KinoDocs.Custom do def new ( ) do frame = Kino.Frame . new ( ) Kino.JS.Live . new ( __MODULE__ , { frame } ) end @impl true def init ( { frame } , ctx ) do { :ok , assign ( ctx , frame : frame ) } end ... end Also see Kino.start_child/1 .","ref":"Kino.JS.Live.html#c:init/2"},{"type":"function","title":"Kino.JS.Live.monitor/1","doc":"Starts monitoring the kino server from the calling process. Refer to Process.monitor/1 for more details.","ref":"Kino.JS.Live.html#monitor/1"},{"type":"function","title":"Kino.JS.Live.new/3","doc":"Instantiates a live JavaScript kino defined by module . The given init_arg is passed to the init/2 callback when the underlying kino process is started. Options :export - a function called to export the given kino to Markdown. This works the same as Kino.JS.new/3 , except the function receives Kino.JS.Live.Context.t/0 as an argument","ref":"Kino.JS.Live.html#new/3"},{"type":"function","title":"Kino.JS.Live.reply/2","doc":"Replies to the kino server. This function can be used to explicitly send a reply to the kino server that called call/3 when the reply cannot be specified in the return value of handle_call/3 . See GenServer.reply/2 for more details.","ref":"Kino.JS.Live.html#reply/2"},{"type":"callback","title":"Kino.JS.Live.terminate/2","doc":"Invoked when the server is about to exit. See GenServer.terminate/2 for more details.","ref":"Kino.JS.Live.html#c:terminate/2"},{"type":"type","title":"Kino.JS.Live.from/0","doc":"","ref":"Kino.JS.Live.html#t:from/0"},{"type":"type","title":"Kino.JS.Live.payload/0","doc":"","ref":"Kino.JS.Live.html#t:payload/0"},{"type":"opaque","title":"Kino.JS.Live.t/0","doc":"","ref":"Kino.JS.Live.html#t:t/0"},{"type":"module","title":"Kino.JS.Live.Context","doc":"State available in Kino.JS.Live server callbacks. Properties :assigns - custom server state kept across callback calls :origin - an opaque identifier of the client that triggered the given action. It is set in Kino.JS.Live.handle_connect/1 and Kino.JS.Live.handle_event/3","ref":"Kino.JS.Live.Context.html"},{"type":"function","title":"Kino.JS.Live.Context.assign/2","doc":"Stores key-value pairs in the state. Examples assign ( ctx , count : 1 , timestamp : DateTime . utc_now ( ) )","ref":"Kino.JS.Live.Context.html#assign/2"},{"type":"function","title":"Kino.JS.Live.Context.broadcast_event/3","doc":"Sends an event to all clients. The event is dispatched to the registered JavaScript callback on all connected clients. Examples broadcast_event ( ctx , &quot;new_point&quot; , %{ x : 10 , y : 10 } )","ref":"Kino.JS.Live.Context.html#broadcast_event/3"},{"type":"function","title":"Kino.JS.Live.Context.emit_event/2","doc":"Emits an event to processes subscribed to this kino. Consumers may subscribe to events emitted by the given instance of Kino.JS.Live using functions in the Kino.Control module, such as Kino.Control.stream/1 . Examples emit_event ( ctx , %{ event : :click , counter : 1 } )","ref":"Kino.JS.Live.Context.html#emit_event/2"},{"type":"function","title":"Kino.JS.Live.Context.send_event/4","doc":"Sends an event to a specific client. The event is dispatched to the registered JavaScript callback on the specific connected client. Examples send_event ( ctx , origin , &quot;new_point&quot; , %{ x : 10 , y : 10 } )","ref":"Kino.JS.Live.Context.html#send_event/4"},{"type":"function","title":"Kino.JS.Live.Context.update/3","doc":"Updates an existing key with the given function in the state. Examples update ( ctx , :count , &amp; ( &amp;1 + 1 ) )","ref":"Kino.JS.Live.Context.html#update/3"},{"type":"type","title":"Kino.JS.Live.Context.origin/0","doc":"","ref":"Kino.JS.Live.Context.html#t:origin/0"},{"type":"type","title":"Kino.JS.Live.Context.t/0","doc":"","ref":"Kino.JS.Live.Context.html#t:t/0"},{"type":"behaviour","title":"Kino.SmartCell","doc":"An interface for defining custom smart cells. A smart cell is a UI wizard designed for producing a piece of code that accomplishes a specific task. In other words, a smart cell is like a code template parameterized through UI interactions. This module builds on top of Kino.JS.Live , consequently keeping all of its component and communication mechanics. The additional callbacks specify how the UI maps to source code. Usage Defining a custom cell is similar to writing a regular Kino.JS.Live component, with a couple specifics. First, we only need to define callbacks, so there is no need for using Kino.JS.Live.new/2 . The Kino.JS.Live.init/2 callback always receives attrs/0 as the first argument. Second, we add a few new bits, namely use Kino.SmartCell and the two corresponding callback definitions. Here is an outline of a custom module defmodule KinoDocs.CustomCell do use Kino.JS use Kino.JS.Live use Kino.SmartCell , name : &quot;Our custom wizard&quot; @impl true def init ( attrs , ctx ) do ... end # Other Kino.JS.Live callbacks ... @impl true def to_attrs ( ctx ) do ... end @impl true def to_source ( attrs ) do ... end end Additionally, in order for Livebook to pick up the custom cell, we need to register our module. This usually happens in application.ex Kino.SmartCell . register ( KinoDocs.CustomCell ) Example As a minimal example, that's how we can define a cell that allows editing the underlying code directly through a textarea. defmodule KinoDocs.SmartCell.Plain do use Kino.JS use Kino.JS.Live use Kino.SmartCell , name : &quot;Plain code editor&quot; @impl true def init ( attrs , ctx ) do source = attrs [ &quot;source&quot; ] || &quot;&quot; { :ok , assign ( ctx , source : source ) } end @impl true def handle_connect ( ctx ) do { :ok , %{ source : ctx . assigns . source } , ctx } end @impl true def handle_event ( &quot;update&quot; , %{ &quot;source&quot; =&gt; source } , ctx ) do broadcast_event ( ctx , &quot;update&quot; , %{ &quot;source&quot; =&gt; source } ) { :noreply , assign ( ctx , source : source ) } end @impl true def to_attrs ( ctx ) do %{ &quot;source&quot; =&gt; ctx . assigns . source } end @impl true def to_source ( attrs ) do attrs [ &quot;source&quot; ] end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, payload) { ctx.importCSS(&quot;main.css&quot;); ctx.root.innerHTML = ` &lt;textarea id=&quot;source&quot;&gt;&lt;/textarea&gt; `; const textarea = ctx.root.querySelector(&quot;#source&quot;); textarea.value = payload.source; textarea.addEventListener(&quot;change&quot;, (event) =&gt; { ctx.pushEvent(&quot;update&quot;, { source: event.target.value }); }); ctx.handleEvent(&quot;update&quot;, ({ source }) =&gt; { textarea.value = source; }); ctx.handleSync(() =&gt; { // Synchronously invokes change listeners document.activeElement &amp;&amp; document.activeElement.dispatchEvent(new Event(&quot;change&quot;)); }); } &quot;&quot;&quot; end asset &quot;main.css&quot; do &quot;&quot;&quot; #source { box-sizing: border-box; width: 100%; min-height: 100px; } &quot;&quot;&quot; end end And then we would register it as Kino.SmartCell . register ( KinoDocs.SmartCell.Plain ) Note that we register a synchronization handler on the client with ctx.handleSync(() =&gt; ...) . This optional handler is invoked before evaluation and it should flush any deferred UI changes to the server. In our example we listen to textarea's &quot;change&quot; event, which is only triggered on blur, so on synchronization we trigger it programmatically. Collaborative editor If a smart cell requires editing some code (like SQL), it may use a dedicated editor instance managed by Livebook. The editor handles syntax highlighting and collaborative editing, similarly to the built-in cells. To enable the editor, we need to include :editor configuration in options returned from the Kino.JS.Live.init/2 callback. @impl true def init ( attrs , ctx ) do # ... { :ok , ctx , editor : [ attribute : &quot;code&quot; , language : &quot;elixir&quot; ] } end Options :attribute - the key to put the source text under in attrs . Required :language - the editor language, used for syntax highlighting. Defaults to nil :placement - editor placement within the smart cell, either :top or :bottom . Defaults to :bottom :default_source - the initial editor source. Defaults to &quot;&quot; Other options Other than the editor configuration, the following options are supported: :reevaluate_on_change - if the cell should be reevaluated whenever the generated source code changes. This option may be helpful in cases where the cell output is a crucial element of the UI interactions. Defaults to false","ref":"Kino.SmartCell.html"},{"type":"function","title":"Kino.SmartCell.definitions/0","doc":"Returns a list of available smart cell definitions.","ref":"Kino.SmartCell.html#definitions/0"},{"type":"function","title":"Kino.SmartCell.prefixed_var_name/2","doc":"Generates unique variable names with the given prefix. When var_name is nil , allocates and returns the next available name. Otherwise, marks the given suffix as taken, provided that var_name has the given prefix. This function can be used to generate default variable names during smart cell initialization, so that don't overlap.","ref":"Kino.SmartCell.html#prefixed_var_name/2"},{"type":"function","title":"Kino.SmartCell.quoted_to_string/1","doc":"Converts the given AST to formatted code string.","ref":"Kino.SmartCell.html#quoted_to_string/1"},{"type":"function","title":"Kino.SmartCell.register/1","doc":"Registers a new smart cell. This should usually be called in application.ex when starting the application. Examples Kino.SmartCell . register ( KinoDocs.CustomCell )","ref":"Kino.SmartCell.html#register/1"},{"type":"callback","title":"Kino.SmartCell.scan_binding/3","doc":"Invoked whenever the base evaluation context changes. This callback receives the binding and environment available to the smart cell code. Note that this callback runs asynchronously and it receives the PID of the smart cell server, so the result needs to be sent explicitly and handled using Kino.JS.Live.handle_info/2 . Important: remember that data sent between processes is copied, so avoid sending large data structures. In particular, when looking at variables, instead of sending their values, extract and send only the relevant metadata. Important: avoid any heavy work in this callback, as it runs in the same process that evaluates code, so we don't want to block it.","ref":"Kino.SmartCell.html#c:scan_binding/3"},{"type":"callback","title":"Kino.SmartCell.scan_eval_result/2","doc":"Invoked when the smart cell code is evaluated. This callback receives the result of an evaluation, either the return value or an exception if raised. This callback runs asynchronously and has the same characteristics as scan_binding/3 .","ref":"Kino.SmartCell.html#c:scan_eval_result/2"},{"type":"callback","title":"Kino.SmartCell.to_attrs/1","doc":"Invoked to compute the smart cell state as serializable attributes.","ref":"Kino.SmartCell.html#c:to_attrs/1"},{"type":"callback","title":"Kino.SmartCell.to_source/1","doc":"Invoked to generate source code based on the given attributes. A list of sources can be returned, in which case the sources are joined into a single source, however converting to a Code cell results in multiple cells.","ref":"Kino.SmartCell.html#c:to_source/1"},{"type":"function","title":"Kino.SmartCell.valid_variable_name?/1","doc":"Checks if the given string is a valid Elixir variable name.","ref":"Kino.SmartCell.html#valid_variable_name?/1"},{"type":"type","title":"Kino.SmartCell.attrs/0","doc":"Attributes are an intermediate form of smart cell state, used to persist and restore cells. Attributes are computed using to_attrs/1 and used to generate the source code using to_source/1 . Note that attributes are serialized and deserialized as JSON for persistence, hence make sure to use JSON-friendly data structures. Persisted attributes are passed to Kino.JS.Live.init/2 as the first argument and should be used to restore the relevant state.","ref":"Kino.SmartCell.html#t:attrs/0"},{"type":"type","title":"Kino.SmartCell.eval_result/0","doc":"","ref":"Kino.SmartCell.html#t:eval_result/0"},{"type":"module","title":"Kino.Inspect","doc":"A struct wrapping any term for default rendering. This is just a meta-struct that implements the Kino.Render protocol, so that the wrapped value is rendered using the inspect protocol.","ref":"Kino.Inspect.html"},{"type":"function","title":"Kino.Inspect.new/1","doc":"Wraps the given term.","ref":"Kino.Inspect.html#new/1"},{"type":"opaque","title":"Kino.Inspect.t/0","doc":"","ref":"Kino.Inspect.html#t:t/0"},{"type":"protocol","title":"Kino.Render","doc":"Protocol defining term formatting in the context of Livebook.","ref":"Kino.Render.html"},{"type":"function","title":"Kino.Render.to_livebook/1","doc":"Transforms the given value into a Livebook-compatible output. For detailed description of the output format see t:Livebook.Runtime.output/0 . When implementing the protocol for custom struct, you generally do not need to worry about the output format. Instead, you can compose built-in kinos and call Kino.Render.to_livebook/1 to get the expected representation. For example, if we wanted to render a custom struct as a mermaid graph, we could do this: defimpl Kino.Render , for : Graph do def to_livebook ( graph ) do source = Graph . to_mermaid ( graph ) mermaid_kino = Kino.Mermaid . new ( source ) Kino.Render . to_livebook ( mermaid_kino ) end end In many cases it is useful to show the default inspect representation alongside our custom visual representation. For this, we can use tabs: defimpl Kino.Render , for : Graph do def to_livebook ( graph ) do source = Graph . to_mermaid ( graph ) mermaid_kino = Kino.Mermaid . new ( source ) inspect_kino = Kino.Inspect . new ( image ) kino = Kino.Layout . tabs ( Graph : mermaid_kino , Raw : inspect_kino ) Kino.Render . to_livebook ( kino ) end end","ref":"Kino.Render.html#to_livebook/1"},{"type":"type","title":"Kino.Render.t/0","doc":"All the types that implement this protocol.","ref":"Kino.Render.html#t:t/0"},{"type":"behaviour","title":"Kino.Table","doc":"A behaviour module for implementing tabular kinos. This module implements table visualization and delegates data fetching and traversal to the behaviour implementation.","ref":"Kino.Table.html"},{"type":"callback","title":"Kino.Table.export_data/2","doc":"Exports the data for download. The returned map must contain the binary, the file extension and the mime type.","ref":"Kino.Table.html#c:export_data/2"},{"type":"callback","title":"Kino.Table.get_data/2","doc":"Loads data matching the given specification.","ref":"Kino.Table.html#c:get_data/2"},{"type":"callback","title":"Kino.Table.init/1","doc":"Invoked once to initialize server state.","ref":"Kino.Table.html#c:init/1"},{"type":"function","title":"Kino.Table.new/3","doc":"Creates a new tabular kino using the given module as data specification. Options :export - a function called to export the given kino to Markdown. This works the same as Kino.JS.new/3 , except the function receives the state as an argument","ref":"Kino.Table.html#new/3"},{"type":"type","title":"Kino.Table.column/0","doc":"","ref":"Kino.Table.html#t:column/0"},{"type":"type","title":"Kino.Table.info/0","doc":"","ref":"Kino.Table.html#t:info/0"},{"type":"type","title":"Kino.Table.rows_spec/0","doc":"","ref":"Kino.Table.html#t:rows_spec/0"},{"type":"type","title":"Kino.Table.state/0","doc":"","ref":"Kino.Table.html#t:state/0"},{"type":"type","title":"Kino.Table.t/0","doc":"","ref":"Kino.Table.html#t:t/0"},{"type":"module","title":"Kino.Test","doc":"Conveniences for testing custom Kino components. In practice, Kino.JS.Live kinos communicate with Livebook via the group leader. During tests, Livebook is out of the equation, so we need to mimic this side of the communication. To do so, add the following setup to your test module: import Kino.Test setup :configure_livebook_bridge","ref":"Kino.Test.html"},{"type":"macro","title":"Kino.Test.assert_broadcast_event/4","doc":"Asserts a Kino.JS.Live kino will broadcast an event within timeout . Examples assert_broadcast_event ( kino , &quot;bump&quot; , %{ by : 2 } )","ref":"Kino.Test.html#assert_broadcast_event/4"},{"type":"macro","title":"Kino.Test.assert_output/2","doc":"Asserts the given output is sent within timeout . Examples assert_output ( { :markdown , &quot;_hey_&quot; } )","ref":"Kino.Test.html#assert_output/2"},{"type":"macro","title":"Kino.Test.assert_output_to/3","doc":"Asserts the given output is sent directly to the given client within timeout . Examples assert_output_to ( &quot;client1&quot; , { :markdown , &quot;_hey_&quot; } )","ref":"Kino.Test.html#assert_output_to/3"},{"type":"macro","title":"Kino.Test.assert_output_to_clients/2","doc":"Asserts the given output is sent directly to all clients within timeout . Examples assert_output_to ( &quot;client1&quot; , { :markdown , &quot;_hey_&quot; } )","ref":"Kino.Test.html#assert_output_to_clients/2"},{"type":"macro","title":"Kino.Test.assert_send_event/4","doc":"Asserts a Kino.JS.Live kino will send an event within timeout to the caller. Examples assert_send_event ( kino , &quot;pong&quot; , %{ } )","ref":"Kino.Test.html#assert_send_event/4"},{"type":"macro","title":"Kino.Test.assert_smart_cell_update/4","doc":"Asserts a smart cell update will be broadcasted within timeout . Matches against the source and attribute that are reported as part of the update. If the source argument is a string, that string is compared in an exact match against the Kino's source. Alternatively, the source argument can be used to bind a variable to the Kino's source, allowing for custom assertions against the source. Examples assert_smart_cell_update ( kino , %{ &quot;variable&quot; =&gt; &quot;x&quot; , &quot;number&quot; =&gt; 10 } , &quot;x = 10&quot; ) assert_smart_cell_update ( kino , %{ &quot;variable&quot; =&gt; &quot;x&quot; , &quot;number&quot; =&gt; 10 } , source ) assert source =~ &quot;10&quot;","ref":"Kino.Test.html#assert_smart_cell_update/4"},{"type":"function","title":"Kino.Test.configure_livebook_bridge/1","doc":"","ref":"Kino.Test.html#configure_livebook_bridge/1"},{"type":"function","title":"Kino.Test.connect/3","doc":"Connects to a Kino.JS.Live kino and returns the initial data. If resolve_fun is given, it runs after sending the connection request and before awaiting for the reply. Examples data = connect ( kino ) assert data == %{ count : 1 }","ref":"Kino.Test.html#connect/3"},{"type":"function","title":"Kino.Test.push_event/3","doc":"Sends a client event to a Kino.JS.Live kino. Examples push_event ( kino , &quot;bump&quot; , %{ &quot;by&quot; =&gt; 2 } )","ref":"Kino.Test.html#push_event/3"},{"type":"function","title":"Kino.Test.start_smart_cell!/2","doc":"Starts a smart cell defined by the given module. Returns a Kino.JS.Live kino for interacting with the cell, as well as the initial source. Examples { kino , source } = start_smart_cell! ( Kino.SmartCell.Custom , %{ &quot;key&quot; =&gt; &quot;value&quot; } )","ref":"Kino.Test.html#start_smart_cell!/2"},{"type":"module","title":"Kino.Audio","doc":"A kino for rendering a binary audio. Examples content = File . read! ( &quot;/path/to/audio.wav&quot; ) Kino.Audio . new ( content , :wav ) content = File . read! ( &quot;/path/to/audio.wav&quot; ) Kino.Audio . new ( content , :wav , autoplay : true , loop : true )","ref":"Kino.Audio.html"},{"type":"function","title":"Kino.Audio.new/3","doc":"Creates a new kino displaying the given binary audio. The given type be either :wav , :mp3 / :mpeg , :ogg or a string with audio MIME type. Options :autoplay - whether the audio should start playing as soon as it is rendered. Defaults to false :loop - whether the audio should loop. Defaults to false :muted - whether the audio should be muted. Defaults to false","ref":"Kino.Audio.html#new/3"},{"type":"type","title":"Kino.Audio.common_audio_type/0","doc":"","ref":"Kino.Audio.html#t:common_audio_type/0"},{"type":"type","title":"Kino.Audio.mime_type/0","doc":"","ref":"Kino.Audio.html#t:mime_type/0"},{"type":"type","title":"Kino.Audio.t/0","doc":"","ref":"Kino.Audio.html#t:t/0"},{"type":"module","title":"Kino.DataTable","doc":"A kino for interactively viewing tabular data. Examples data = [ %{ id : 1 , name : &quot;Elixir&quot; , website : &quot;https://elixir-lang.org&quot; } , %{ id : 2 , name : &quot;Erlang&quot; , website : &quot;https://www.erlang.org&quot; } ] Kino.DataTable . new ( data ) The tabular view allows you to quickly preview the data and analyze it thanks to sorting capabilities. data = for pid &lt;- Process . list ( ) do pid |&gt; Process . info ( ) |&gt; Keyword . merge ( registered_name : nil ) end Kino.DataTable . new ( data , keys : [ :registered_name , :initial_call , :reductions , :stack_size ] )","ref":"Kino.DataTable.html"},{"type":"function","title":"Kino.DataTable.new/2","doc":"Creates a new kino displaying given tabular data. Options :keys - a list of keys to include in the table for each record. The order is reflected in the rendered table. Optional :name - The displayed name of the table. Defaults to &quot;Data&quot; :sorting_enabled - whether the table should support sorting the data. Sorting requires traversal of the whole enumerable, so it may not be desirable for large lazy enumerables. Defaults to true","ref":"Kino.DataTable.html#new/2"},{"type":"type","title":"Kino.DataTable.t/0","doc":"","ref":"Kino.DataTable.html#t:t/0"},{"type":"module","title":"Kino.Download","doc":"A kino for downloading file content. Examples Kino.Download . new ( fn -&gt; &quot;Example text&quot; end ) Kino.Download . new ( fn -&gt; Jason . encode! ( %{ &quot;foo&quot; =&gt; &quot;bar&quot; } ) end , filename : &quot;data.json&quot; ) Kino.Download . new ( fn -&gt; &lt;&lt; 0 , 1 &gt;&gt; end , filename : &quot;data.bin&quot; , label : &quot;Binary data&quot; )","ref":"Kino.Download.html"},{"type":"function","title":"Kino.Download.new/2","doc":"Creates a button for file download. The given function is invoked to generate the file content whenever a download is requested. Options :filename - the default filename suggested for download. Defaults to &quot;download&quot; :label - the button text. Defaults to the value of :filename if present and &quot;Download&quot; otherwise","ref":"Kino.Download.html#new/2"},{"type":"type","title":"Kino.Download.t/0","doc":"","ref":"Kino.Download.html#t:t/0"},{"type":"module","title":"Kino.ETS","doc":"A kino for interactively viewing an ETS table. Examples tid = :ets . new ( :users , [ :set , :public ] ) Kino.ETS . new ( tid ) Kino.ETS . new ( :elixir_config )","ref":"Kino.ETS.html"},{"type":"function","title":"Kino.ETS.new/1","doc":"Creates a new kino displaying the given ETS table. Note that private tables cannot be read by an arbitrary process, so the given table must have either public or protected access.","ref":"Kino.ETS.html#new/1"},{"type":"type","title":"Kino.ETS.t/0","doc":"","ref":"Kino.ETS.html#t:t/0"},{"type":"module","title":"Kino.HTML","doc":"A kino for rendering HTML content. The HTML may include &lt;script&gt; tags with global JS to be executed. In case you need to parameterize the HTML with dynamic values, write a custom Kino.JS component. Examples Kino.HTML . new ( &quot;&quot;&quot; &lt;h3&gt;Look!&lt;/h3&gt; &lt;p&gt;I wrote this HTML from &lt;strong&gt;Kino&lt;/strong&gt;!&lt;/p&gt; &quot;&quot;&quot; ) Kino.HTML . new ( &quot;&quot;&quot; &lt;button id=&quot;button&quot;&gt;Click&lt;/button&gt; &lt;script&gt; const button = document.querySelector(&quot;#button&quot;); button.addEventListener(&quot;click&quot;, (event) =&gt; { button.textContent = &quot;Clicked!&quot; }); &lt;/script&gt; &quot;&quot;&quot; )","ref":"Kino.HTML.html"},{"type":"function","title":"Kino.HTML.new/1","doc":"Creates a new kino displaying the given HTML.","ref":"Kino.HTML.html#new/1"},{"type":"type","title":"Kino.HTML.t/0","doc":"","ref":"Kino.HTML.html#t:t/0"},{"type":"module","title":"Kino.Image","doc":"A kino for rendering a binary image. This is just a meta-struct that implements the Kino.Render protocol, so that it gets rendered as the underlying image. Examples content = File . read! ( &quot;/path/to/image.jpeg&quot; ) Kino.Image . new ( content , &quot;image/jpeg&quot; )","ref":"Kino.Image.html"},{"type":"function","title":"Kino.Image.new/1","doc":"Creates a new kino similarly to new/2 from a compatible term. Currently the supported terms are: Nx.Tensor in HWC order","ref":"Kino.Image.html#new/1"},{"type":"function","title":"Kino.Image.new/2","doc":"Creates a new kino displaying the given binary image. The given type be either :jpeg / :jpg , :png , :gif , :svg , :pixel or a string with image MIME type. Pixel data Note that a special image/x-pixel MIME type is supported. The binary consists of the following consecutive parts: height - 32 bits (unsigned big-endian integer) width - 32 bits (unsigned big-endian integer) channels - 8 bits (unsigned integer) data - pixel data in HWC order Pixel data consists of 8-bit unsigned integers. The number of channels can be either: 1 (grayscale), 2 (grayscale + alpha), 3 (RGB), or 4 (RGB + alpha).","ref":"Kino.Image.html#new/2"},{"type":"type","title":"Kino.Image.common_image_type/0","doc":"","ref":"Kino.Image.html#t:common_image_type/0"},{"type":"type","title":"Kino.Image.mime_type/0","doc":"","ref":"Kino.Image.html#t:mime_type/0"},{"type":"opaque","title":"Kino.Image.t/0","doc":"","ref":"Kino.Image.html#t:t/0"},{"type":"module","title":"Kino.Layout","doc":"Layout utilities for arranging multiple kinos together.","ref":"Kino.Layout.html"},{"type":"function","title":"Kino.Layout.grid/2","doc":"Arranges outputs into a grid. Note that the grid does not support scrolling, it always squeezes the content, such that it does not exceed the page width. Options :columns - the number of columns in the grid. Defaults to 1 :boxed - whether the grid should be wrapped in a bordered box. Defaults to false :gap - the amount of spacing between grid items in pixels. Defaults to 8 Examples images = for path &lt;- paths do path |&gt; File . read! ( ) |&gt; Kino.Image . new ( :jpeg ) end Kino.Layout . grid ( images , columns : 3 )","ref":"Kino.Layout.html#grid/2"},{"type":"function","title":"Kino.Layout.tabs/1","doc":"Arranges outputs into separate tabs. Examples data = [ %{ id : 1 , name : &quot;Elixir&quot; , website : &quot;https://elixir-lang.org&quot; } , %{ id : 2 , name : &quot;Erlang&quot; , website : &quot;https://www.erlang.org&quot; } ] Kino.Layout . tabs ( [ Table : Kino.DataTable . new ( data ) , Raw : data ] )","ref":"Kino.Layout.html#tabs/1"},{"type":"opaque","title":"Kino.Layout.t/0","doc":"","ref":"Kino.Layout.html#t:t/0"},{"type":"module","title":"Kino.Markdown","doc":"A kino for rendering Markdown content. This is just a meta-struct that implements the Kino.Render protocol, so that it gets rendered as markdown. Examples Kino.Markdown . new ( &quot;&quot;&quot; # Example A regular Markdown file. ## Code ```elixir &quot;Elixir&quot; |&gt; String.graphemes() |&gt; Enum.frequencies() ``` ## Table | ID | Name | Website | | -- | ------ | ----------------------- | | 1 | Elixir | https://elixir-lang.org | | 2 | Erlang | https://www.erlang.org | &quot;&quot;&quot; ) This format may come in handy when exploring Markdown from external sources: text = File . read! ( &quot;/path/to/README.md&quot; ) Kino.Markdown . new ( text )","ref":"Kino.Markdown.html"},{"type":"function","title":"Kino.Markdown.new/2","doc":"Creates a new kino displaying the given Markdown content. Options :chunk - whether this is a part of a larger text. Adjacent chunks are merged into a single text. This is useful for streaming content. Defaults to false","ref":"Kino.Markdown.html#new/2"},{"type":"opaque","title":"Kino.Markdown.t/0","doc":"","ref":"Kino.Markdown.html#t:t/0"},{"type":"module","title":"Kino.Mermaid","doc":"A kino for rendering Mermaid graphs. Relation to Kino.Markdown Mermaid graphs can also be generated dynamically with Kino.Markdown , however the output of Kino.Markdown is never persisted in the notebook source. Kino.Mermaid doesn't have this limitation. Examples Kino.Mermaid . new ( &quot;&quot;&quot; graph TD; A--&gt;B; A--&gt;C; B--&gt;D; C--&gt;D; &quot;&quot;&quot; )","ref":"Kino.Mermaid.html"},{"type":"function","title":"Kino.Mermaid.new/1","doc":"Creates a new kino displaying the given Mermaid graph.","ref":"Kino.Mermaid.html#new/1"},{"type":"type","title":"Kino.Mermaid.t/0","doc":"","ref":"Kino.Mermaid.html#t:t/0"},{"type":"module","title":"Kino.Text","doc":"A kino for rendering text content. For rich text, use Kino.Markdown . Examples Kino.Text . new ( &quot;Hello!&quot; ) [ :green , &quot;Hello!&quot; ] |&gt; IO.ANSI . format ( ) |&gt; IO . iodata_to_binary ( ) |&gt; Kino.Text . new ( terminal : true )","ref":"Kino.Text.html"},{"type":"function","title":"Kino.Text.new/2","doc":"Creates a new kino displaying the given text content. Options :terminal - whether to render the text as if it were printed to standard output, supporting ANSI escape codes. Defaults to false :chunk - whether this is a part of a larger text. Adjacent chunks are merged into a single text. This is useful for streaming content. Defaults to false","ref":"Kino.Text.html#new/2"},{"type":"opaque","title":"Kino.Text.t/0","doc":"","ref":"Kino.Text.html#t:t/0"},{"type":"module","title":"Kino.Tree","doc":"A kino for interactively viewing nested data as a tree view. The data can be any term. Examples data = %{ id : 1 , email : &quot;user@example.com&quot; , inserted_at : ~U[2022-01-01T10:00:00Z] , addresses : [ %{ country : &quot;pl&quot; , city : &quot;Kraków&quot; , street : &quot;Karmelicka&quot; , zip : &quot;00123&quot; } ] } Kino.Tree . new ( data ) The tree view is particularly useful when inspecting larger data structures: data = Process . info ( self ( ) ) Kino.Tree . new ( data )","ref":"Kino.Tree.html"},{"type":"function","title":"Kino.Tree.new/1","doc":"Creates a new kino displaying the given data structure.","ref":"Kino.Tree.html#new/1"},{"type":"module","title":"Kino.Video","doc":"A kino for rendering a binary video. Examples content = File . read! ( &quot;/path/to/video.mp4&quot; ) Kino.Video . new ( content , :mp4 ) content = File . read! ( &quot;/path/to/video.mp4&quot; ) Kino.Video . new ( content , :mp4 , autoplay : true , loop : true )","ref":"Kino.Video.html"},{"type":"function","title":"Kino.Video.new/3","doc":"Creates a new kino displaying the given binary video. The given type be either :mp4 , :ogg , :avi , :wmv , :mov or a string with video MIME type. Options :autoplay - whether the video should start playing as soon as it is rendered. Defaults to false :loop - whether the video should loop. Defaults to false :muted - whether the video should be muted. Defaults to false","ref":"Kino.Video.html#new/3"},{"type":"type","title":"Kino.Video.common_video_type/0","doc":"","ref":"Kino.Video.html#t:common_video_type/0"},{"type":"type","title":"Kino.Video.mime_type/0","doc":"","ref":"Kino.Video.html#t:mime_type/0"},{"type":"type","title":"Kino.Video.t/0","doc":"","ref":"Kino.Video.html#t:t/0"},{"type":"exception","title":"Kino.FS.ForbiddenError","doc":"Exception raised when access to a notebook file is forbidden.","ref":"Kino.FS.ForbiddenError.html"},{"type":"exception","title":"Kino.InterruptError","doc":"Exception raised to stop evaluation in expected manner.","ref":"Kino.InterruptError.html"},{"type":"type","title":"Kino.InterruptError.t/0","doc":"","ref":"Kino.InterruptError.html#t:t/0"}]